//
//  Master Timer Object
//
//  Usage:
//      masterTimer.setInterval(name.namespace, timeout, function() { });
//      masterTimer.clearInterval(name.namespace);
//
//  Namespace is optional and allows for mass-clearing e.g.:
//      masterTimer.clearIntervals(namespace);
//

function masterTimerObj()
{
    this.worker = null;
    this.rate = null;
    this.intervals = null;

    this.init();

    this.begin();
}

//
//  Initialize the web worker with onerror and onmessage functions. Reset master timer intervals array.
//
masterTimerObj.prototype.init = function()
{
    var me = this;
    me.worker = new Worker(DIR_ROOT+"/www/js/masterTimerWorker.js");
    me.worker.onerror = function(err) { alert("Master Timer Worker Error: "+err.message); }
    me.worker.onmessage = function(e) { me.parseWorkerMessage(e.data); };
    me.intervals = new Array();
    me.worker.postMessage("init");
    me.setRate(100);                  // <-- Shortest timer supported by master timer.
}

masterTimerObj.prototype.setRate = function(rate) { this.rate = rate; this.worker.postMessage(["rate", rate]); }
masterTimerObj.prototype.begin = function() { this.worker.postMessage("begin"); }
masterTimerObj.prototype.pause = function() { this.worker.postMessage("pause"); }
masterTimerObj.prototype.die = function() { this.worker.terminate(); }
masterTimerObj.prototype.clearInterval = function(intervalName) { this.worker.postMessage(["remove", intervalName]); }
masterTimerObj.prototype.clearIntervals = function(namespace) { this.worker.postMessage(["clearNamespace", namespace]); }

//
//  Add a new interval to master timer. Only allow intervals with unique names and evenly-divisible rates (based on master timer rate).
//
masterTimerObj.prototype.setInterval = function(intervalName, intervalMS, whatToDo)
{
    var me = this;
    var counter = { name: intervalName, rate: intervalMS, count: 0 };

    if(intervalMS%me.rate != 0)
    {
        alert("ERROR - only add intervals to master timer with a rate divisible by "+me.rate+"ms");
        return;
    }

    for(var i=0;i<me.intervals.length;i++)
        if(me.intervals[i].name == intervalName)
        {
            alert("ERROR - an interval with the name "+intervalName+" already exists!");
            return;
        }

    me.intervals.push({name: intervalName, rate: intervalMS, callback: whatToDo});
    me.worker.postMessage(["add", counter]);
}

//
//  When an interval is cleared by the worker, mirror that change here.
//
masterTimerObj.prototype.intervalCleared = function(intervalName)
{
    var me = this;

    for(var i=0;i<me.intervals.length;i++)
        if(me.intervals[i].name == intervalName)
        {
            me.intervals.splice(i, 1);
            return;
        }
}

//
//  Handle the messages sent back from the worker.
//
masterTimerObj.prototype.parseWorkerMessage = function(data)
{
    var m = (typeof(data) == "string") ? data : data[0];

    switch(m)
    {
        case "tick":
            this.tick(data[1]);
            break;

        case "removed":
            this.intervalCleared(data[1]);
            break;

        case "dead":
            this.end();
            break;

        default:
            alert("--- "+data+" ---");
            break;
    }
}

//
//  Called when the worker determines that a particular interval has expired. Call that intervals callback() function.
//
masterTimerObj.prototype.tick = function(data)
{
    var me = this;

    for(var i=0;i<me.intervals.length;i++)
        if(me.intervals[i].name == data.name)
        {
            me.intervals[i].callback();
            return;
        }
}

